@@ -1,7 +1,10 @@
-import logging
+import logging, os
 from fastmcp import FastMCP
-from agentmake import agentmake
-from biblemate import AGENTMAKE_CONFIG
+from fastmcp.prompts.prompt import PromptMessage, TextContent
+from agentmake import agentmake, getDictionaryOutput, DEFAULT_AI_BACKEND
+from biblemate import AGENTMAKE_CONFIG, config
+from agentmake.plugins.uba.lib.BibleBooks import BibleBooks
+from biblemate.core.bible_db import BibleVectorDatabase
 
 # Configure logging before creating the FastMCP server
 logging.basicConfig(format="[%(levelname)s]: %(message)s", level=logging.ERROR)
@@ -11,6 +14,47 @@ mcp = FastMCP(name="BibleMate AI")
 def getResponse(messages:list) -> str:
     return messages[-1].get("content") if messages and "content" in messages[-1] else "Error!"
 
+@mcp.tool
+def search_bible(request:str) -> str:
+    """search the bible; search string must be given"""
+    bible_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), "data", "bibles", f"{config.default_bible}.bible")
+    if os.path.isfile(bible_file):
+        # extract the search string
+        schema = {
+            "name": "search_google",
+            "description": "search the bible; search string must be given",
+            "parameters": {
+                "type": "object",
+                "properties": {
+                    "search_string": {
+                        "type": "string",
+                        "description": "search string for searching the bible",
+                    },
+                },
+                "required": ["search_string"],
+            },
+        }
+        search_string = getDictionaryOutput(request, schema=schema, backend=DEFAULT_AI_BACKEND)["search_string"]
+        # perform the searches
+        abbr = BibleBooks.abbrev["eng"]
+        db = BibleVectorDatabase(bible_file)
+        exact_matches = [f"({abbr[str(b)]} {c}:{v}) {content}" for b, c, v, content in db.search_verses_partial(search_string)]
+        if os.path.getsize(bible_file) > 380000000:
+            semantic_matches = [f"({abbr[str(b)]} {c}:{v}) {content}" for b, c, v, content in db.search_meaning(search_string)]
+        else:
+            semantic_matches = []
+        output = f"""# Bible Search Results
+
+## Exact Matches ({len(exact_matches)} verse(s))
+
+{"\n- ".join(exact_matches)}
+
+## Semantic Matches ({len(semantic_matches)} verse(s))
+
+{"\n- ".join(semantic_matches) if semantic_matches else "[`Ollama` is not found! BibleMate AI uses `Ollama` to generate embeddings for semantic searches. You may install it from https://ollama.com/ so that you can perform semantic searches of the Bible with BibleMate AI.]"}"""
+        return output
+    return ""
+
 @mcp.tool
 def compare_bible_translations(request:str) -> str:
     """compare Bible translations; bible verse reference(s) must be given"""
@@ -32,18 +76,32 @@ def retrieve_bible_cross_references(request:str) -> str:
     messages = agentmake(request, **{'input_content_plugin': 'uba/every_single_ref', 'tool': 'uba/xref'}, **AGENTMAKE_CONFIG)
     return getResponse(messages)
 
+@mcp.tool
+def retrieve_pointed_hebrew_or_accented_greek_bible_verses(request:str) -> str:
+    """retrieve Hebrew (with pointed vowels) or Greek (with accents) Bible verses; bible verse reference(s) must be given"""
+    global agentmake, getResponse
+    messages = agentmake(request, **{'tool': 'uba/ohgb'}, **AGENTMAKE_CONFIG)
+    return getResponse(messages)
+
+@mcp.tool
+def retrieve_hebrew_or_greek_bible_verses(request:str) -> str:
+    """retrieve Hebrew (without pointed vowels) or Greek (without accents) Bible verses; bible verse reference(s) must be given"""
+    global agentmake, getResponse
+    messages = agentmake(request, **{'tool': 'uba/mob'}, **AGENTMAKE_CONFIG)
+    return getResponse(messages)
+
 @mcp.tool
 def retrieve_english_bible_verses(request:str) -> str:
-    """retrieve English Bible verses; bible verse reference(s) must be given"""
+    """retrieve English Bible verses; bible verse reference(s) must be given, e.g. John 3:16-17; multiple references accepted, e.g. John 3:16; Deut 6:4"""
     global agentmake, getResponse
     messages = agentmake(request, **{'tool': 'uba/net'}, **AGENTMAKE_CONFIG)
     return getResponse(messages)
 
 @mcp.tool
-def search_bible_or_run_uba_command(request:str) -> str:
-    """search the bible; run UniqueBible App UBA command; either search string or full command must be given"""
+def retrieve_english_bible_chapter(request:str) -> str:
+    """retrieve English Bible chapter, a whole chapter; bible chapter reference must be given, e.g. John 3"""
     global agentmake, getResponse
-    messages = agentmake(request, **{'tool': 'uba/cmd'}, **AGENTMAKE_CONFIG)
+    messages = agentmake(request, **{'tool': 'uba/net_chapter'}, **AGENTMAKE_CONFIG)
     return getResponse(messages)
 
 @mcp.tool
@@ -333,4 +391,44 @@ def write_bible_sermon(request:str) -> str:
     messages = agentmake(request, **{'instruction': 'bible/sermon', 'system': 'auto'}, **AGENTMAKE_CONFIG)
     return getResponse(messages)
 
+@mcp.prompt
+def simple_bible_study(request:str) -> PromptMessage:
+    """Perform a simple bible study task"""
+    global PromptMessage, TextContent
+    prompt_text = f"""You are a bible study agent. You check the user request, under the `User Request` section, and resolve it with the following steps in order:
+1. Call tool 'retrieve_english_bible_verses' for Bible text, 
+2. Call tool 'retrieve_bible_cross_references' for Bible cross-references, 
+3. Call tool 'study_old_testament_themes' for study old testament themes or 'study_new_testament_themes' for study old testament themes, and 
+4. Call tool 'write_bible_theology' to explain its theology.
+
+# User Request
+
+---
+{request}
+---
+"""
+    return PromptMessage(role="user", content=TextContent(type="text", text=prompt_text))
+
+@mcp.prompt
+def bible_devotion(request:str) -> PromptMessage:
+    """Write bible devotion based on user content"""
+    global PromptMessage, TextContent
+    prompt_text = f"""
+You are a bible devotional agent. You check the user content, under the `User Content` section, and write a devotional about it with the following steps in order:
+
+1. Analyze the themes using @study_new_testament_themes for new testament passages or @study_old_testament_themes for old testament passages.
+2. Identify and explain key biblical keywords from the passage using @identify_bible_keywords.
+3. Write insights for the devotion using @write_bible_insights.
+4. Relate the passage to daily life using @write_bible_applications.
+5. Compose a touching devotion using @write_bible_devotion.
+Ensure each step is clearly addressed and the final output is cohesive and inspiring.
+
+# User Content
+
+---
+{request}
+---
+"""
+    return PromptMessage(role="user", content=TextContent(type="text", text=prompt_text))
+
 mcp.run(show_banner=False)
\ No newline at end of file
